/*
    Project: TCGINA
    Version: V1.18
    File   : tcgmisc.c

    Copyright (c) 2005-2008 Author of TCGINA DLL. All rights reserved.
    Public Key Fingerprint: 294B A769 4A0A CC05 DAE6 00DD FF47 8C72 4097 67CE

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include "tcgpriv.h"
#include <Winnetwk.h>
#include <tchar.h>

#ifdef TC_DEBUG
static void TC_DbgPrintMultiString(LPTSTR pszFormat, LPCTSTR pszStrings)
{
    while (*pszStrings) {
        TC_DbgPrintf(pszFormat, pszStrings);
        pszStrings += _tcslen(pszStrings) + 1;
    }
}
#endif

#define ST_GENERAL        0
#define ST_USER           1
#define ST_REMOTE_SESSION 2

void WINAPI TCG_ReadSettings(PTCG_SETTINGS pSettings,
  LPCTSTR pszUserName)
{
    HKEY  hKey;
    TCHAR szRegKeyName[MAX_PATH];
    int   iType;

    TC_Assert(pSettings);

    if (pszUserName == NULL) {
        iType = ST_GENERAL;
    } else {
        iType = (_tcschr(pszUserName, TEXT('\\')) == NULL) ?
          ST_USER : ST_REMOTE_SESSION;
    }

    if (iType == ST_USER) {
        TCG_ReadSettings(pSettings, NULL);
    }

    _tcscpy(szRegKeyName, TCG_REGKEY_NAME);
    if (iType == ST_GENERAL) {
        /* Initialize settings with default values */
        TC_ZeroMemory(pSettings, sizeof(TCG_SETTINGS));
    } else {
        if (!TC_AppendString(szRegKeyName, countof(szRegKeyName), TEXT("\\"))) {
            TC_DbgPutsA("Cannot append backslash to registry key name!\n");
            return;
        }
        if (!TC_AppendString(szRegKeyName, countof(szRegKeyName),
          pszUserName)) {
            TC_DbgPutsA("Cannot append user name to registry key name!\n");
            return;
        }
    }

    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, szRegKeyName, 0, KEY_READ, &hKey) ==
      ERROR_SUCCESS) {
#ifdef SUPPORT_KEYFILES
        DWORD dw;
#endif
        TC_ReadRegistryInteger(hKey, TEXT("TCOptions"),
          &pSettings->dwTCOptions);
        TC_ReadRegistryInteger(hKey, TEXT("TCMountOptions"),
          &pSettings->dwTCMountOptions);
        TC_ReadRegistryInteger(hKey, TEXT("TCPasswordTimeout"),
          &pSettings->dwTCPasswordTimeout);
        TC_ReadRegistryInteger(hKey, TEXT("TCPasswordTotalTimeout"),
          &pSettings->dwTCPasswordTotalTimeout);
        TC_ReadRegistryInteger(hKey, TEXT("TCKeyFileTimeout"),
          &pSettings->dwTCKeyFileTimeout);

        TC_ReadRegistryStringList(hKey, TEXT("TCPath"), pSettings->szTCPath,
          countof(pSettings->szTCPath));
        if (TC_ReadRegistryString(hKey, TEXT("TCFileName"),
          pSettings->szTCFileName,
          countof(pSettings->szTCFileName))) {
            if (*pSettings->szTCFileName) {
                pSettings->fTCFileNameAvailable = TRUE;
            }
        }
        TC_ReadRegistryString(hKey, TEXT("TCDrive"), pSettings->szTCDrive,
          countof(pSettings->szTCDrive));
        TC_ReadRegistryString(hKey, TEXT("TCFirstAutoMountDrive"),
          pSettings->szTCFirstAutoMountDrive,
          countof(pSettings->szTCFirstAutoMountDrive));
        TC_ReadRegistryStringList(hKey, TEXT("TCKeyFileNames"),
          pSettings->szTCKeyFileNames,
          countof(pSettings->szTCKeyFileNames));
        TC_ReadRegistryString(hKey, TEXT("TCKeyFileDrives"),
          pSettings->szTCKeyFileDrives,
          countof(pSettings->szTCKeyFileDrives));

#ifdef SUPPORT_TCPASSWORD
        TC_ReadRegistryString(hKey, TEXT("TCPassword"),
          pSettings->szTCPassword,
          countof(pSettings->szTCPassword));
#endif

#ifdef SUPPORT_KEYFILES
        TC_ReadRegistryString(hKey, TEXT("KFFileName"),
          pSettings->szKFFileName,
          countof(pSettings->szKFFileName));
        TC_ReadRegistryString(hKey, TEXT("KFDrives"),
          pSettings->szKFDrives,
          countof(pSettings->szKFDrives));
        TC_ReadRegistryString(hKey, TEXT("KFPassword"),
          pSettings->szKFPassword,
          countof(pSettings->szKFPassword));
        if (TC_ReadRegistryInteger(hKey, TEXT("KFFileOffset"),
          &dw)) {
            pSettings->liKFFileOffset.LowPart = dw;
        }
        if (TC_ReadRegistryInteger(hKey, TEXT("KFFileOffsetHigh"),
          &dw)) {
            pSettings->liKFFileOffset.HighPart = dw;
        }
        TC_ReadRegistryInteger(hKey, TEXT("KFFileSize"),
          &pSettings->dwKFFileSize);
        TC_ReadRegistryInteger(hKey, TEXT("KFID"),
          &pSettings->dwKFID);
        TC_ReadRegistryInteger(hKey, TEXT("KFNoPasswordDialog"),
          &pSettings->dwKFNoPasswordDialog);
        TC_ReadRegistryInteger(hKey, TEXT("KFMediumTimeout"),
          &pSettings->dwKFMediumTimeout);
        TC_ReadRegistryInteger(hKey, TEXT("KFPasswordTimeout"),
          &pSettings->dwKFPasswordTimeout);
        TC_ReadRegistryInteger(hKey, TEXT("KFPasswordTotalTimeout"),
          &pSettings->dwKFPasswordTotalTimeout);
#endif

#ifdef SUPPORT_NETWORK
        TC_ReadRegistryString(hKey, TEXT("NDLocalName"),
          pSettings->szNDLocalName,
          countof(pSettings->szNDLocalName));
        TC_ReadRegistryString(hKey, TEXT("NDRemoteName"),
          pSettings->szNDRemoteName,
          countof(pSettings->szNDRemoteName));
        TC_ReadRegistryString(hKey, TEXT("NDUserName"),
          pSettings->szNDUserName,
          countof(pSettings->szNDUserName));
        TC_ReadRegistryString(hKey, TEXT("NDPassword"),
          pSettings->szNDPassword,
          countof(pSettings->szNDPassword));
#endif

#ifdef SUPPORT_SERVICES
        TC_ReadRegistryStringList(hKey, TEXT("TCStartServices"),
          pSettings->szTCStartServices,
          countof(pSettings->szTCStartServices));
        TC_ReadRegistryStringList(hKey, TEXT("TCStopServices"),
          pSettings->szTCStopServices,
          countof(pSettings->szTCStartServices));
#endif

        RegCloseKey(hKey);
    }

    pSettings->dwTCOptions      &= TC_DISPLAY_PASSWORD |
                                   TC_IGNORE_USER_ACCOUNT_PASSWORD |
                                   TC_AUTOMOUNT_VOLUME_IS_MANDATORY |
                                   TC_DISABLE_WIPING_ON_LOGOFF |
                                   TC_DISABLE_DISMOUNTING_ON_LOGOFF;
    pSettings->dwTCMountOptions &= TC_CACHE_PASSWORDS |
                                   TC_READ_ONLY |
                                   TC_REMOVABLE |
                                   TC_SHARED |
                                   TC_UPDATE_TIMESTAMPS;
    pSettings->dwTCMountOptions |= TC_PERSISTENT;

    if (_tcschr(pSettings->szTCFileName, TEXT('\\'))) {
        *pSettings->szTCPath = TEXT('\0');
    } else if (*pSettings->szTCPath == TEXT('\0')) {
        TC_GetProfilesDirectory(pSettings->szTCPath,
          countof(pSettings->szTCPath));
    }

    if (*pSettings->szTCFileName == TEXT('\0')) {
        _tcscpy(pSettings->szTCFileName, TEXT("%USERNAME%.tc"));
    }
    if (*pSettings->szTCDrive == TEXT('\0')) {
        _tcscpy(pSettings->szTCDrive, TEXT("U:\\"));
    }

    if (iType == ST_USER) {

        if (GetSystemMetrics(SM_REMOTESESSION)) {

            TCHAR szRemoteName[MAX_PATH];

            if (TC_CopyString(szRemoteName, countof(szRemoteName),
              pszUserName) &&
              TC_AppendString(szRemoteName, countof(szRemoteName),
              TEXT("\\RemoteSession"))) {
                TCG_ReadSettings(pSettings, szRemoteName);
            }
        }

        TC_ReplaceString(pSettings->szTCPath,
          countof(pSettings->szTCPath),
          TEXT("%USERNAME%"), pszUserName);

        TC_ReplaceString(pSettings->szTCFileName,
          countof(pSettings->szTCFileName),
          TEXT("%USERNAME%"), pszUserName);

#ifdef TC_DEBUG
        TC_DbgPrintf(TEXT("Settings for user \"%s\":\n"), pszUserName);
        TC_DbgPrintfA(    "  TCOptions               = 0x%08x\n"
                          "  TCMountOptions          = 0x%08x\n"
                          "  TCPasswordTimeout       = %u\n"
                          "  TCKeyFileTimeout        = %u\n",
                          pSettings->dwTCOptions,
                          pSettings->dwTCMountOptions,
                          pSettings->dwTCPasswordTimeout,
                          pSettings->dwTCKeyFileTimeout);
        TC_DbgPrintMultiString(
                     TEXT("  TCPath                  = %s\n"),
                     pSettings->szTCPath);
        if (pSettings->fTCFileNameAvailable) {
            TC_DbgPrintf(TEXT("  TCFileName              = %s\n"),
              pSettings->szTCFileName);
        } else {
            TC_DbgPrintf(TEXT("  TCFileName              = n/a (%s)\n"),
              pSettings->szTCFileName);
        }
        TC_DbgPrintf(TEXT("  TCDrive                 = %s\n"),
          pSettings->szTCDrive);
        TC_DbgPrintf(TEXT("  TCFirstAutoMountDrive   = %s\n"),
          pSettings->szTCFirstAutoMountDrive);
        TC_DbgPrintf(TEXT("  TCKeyFileDrives         = %s\n"),
          pSettings->szTCKeyFileDrives);
        TC_DbgPrintMultiString(
                     TEXT("  TCKeyFileName           = %s\n"),
                     pSettings->szTCKeyFileNames);

#ifdef SUPPORT_KEYFILES
        TC_DbgPrintf(TEXT("  KFFileName              = %s\n"),
          pSettings->szKFFileName);
        TC_DbgPrintf(TEXT("  KFDrives                = %s\n"),
          pSettings->szKFDrives);
        TC_DbgPrintfA(    "  KFPassword              = (%s)\n",
            *pSettings->szKFPassword ? "non-empty" : "empty");
        TC_DbgPrintfA(    "  KFFileOffset            = 0x%08x\n"
                          "  KFFileOffsetHigh        = 0x%08x\n"
                          "  KFFileSize              = %u\n"
                          "  KFID                    = 0x%08X\n"
                          "  KFNoPasswordDialog      = %u\n"
                          "  KFMediumTimeout         = %u\n"
                          "  KFPasswordTimeout       = %u\n",
                          pSettings->liKFFileOffset.LowPart,
                          pSettings->liKFFileOffset.HighPart,
                          pSettings->dwKFFileSize,
                          pSettings->dwKFID,
                          pSettings->dwKFNoPasswordDialog,
                          pSettings->dwKFMediumTimeout,
                          pSettings->dwKFPasswordTimeout);
#endif

#ifdef SUPPORT_NETWORK
        TC_DbgPrintf(TEXT("  NDLocalName             = %s\n"),
          pSettings->szNDLocalName);
        TC_DbgPrintf(TEXT("  NDRemoteName            = %s\n"),
          pSettings->szNDRemoteName);
        TC_DbgPrintf(TEXT("  NDUserName              = %s\n"),
          pSettings->szNDUserName);
        TC_DbgPrintfA(    "  NDPassword              = (%s)\n",
            *pSettings->szNDPassword ? "non-empty" : "empty");
        TC_DbgPrintMultiString(
                     TEXT("  TCStopServices          = (%s)\n"),
                     pSettings->szTCStopServices);
        TC_DbgPrintMultiString(
                     TEXT("  TCStartServices         = (%s)\n"),
                     pSettings->szTCStartServices);
#endif

#endif
    }
}

/*-----------------------------------------------------------------------------
// Network functions                                                         */

#ifdef SUPPORT_NETWORK

DWORD WINAPI TCG_MapNetworkDrive(LPTSTR pszLocalName,
  LPTSTR pszRemoteName, LPCTSTR pszUserName, LPCTSTR pszPassword)
{
    /*-------------------------------------------------------------------------
    // References:
    //
    // http://support.microsoft.com/default.aspx?scid=kb;en-us;Q180362
    //-----------------------------------------------------------------------*/

    TCHAR       szDomainAndUserName[MAX_PATH];
    NETRESOURCE nr;
    DWORD       dwErrorCode;

    TC_Assert(pszLocalName);
    TC_Assert(pszRemoteName);
    TC_Assert(pszUserName);
    TC_Assert(pszPassword);

    if (*pszLocalName == TEXT('\0')) return NO_ERROR;

    if (_tcschr(pszUserName, TEXT('\\')) == 0) {
        if (TC_CopyString(szDomainAndUserName, MAX_PATH, pszRemoteName+2)) {
            LPTSTR pszTemp = _tcschr(szDomainAndUserName, TEXT('\\'));
            if (pszTemp) {
                pszTemp[1] = TEXT('\0');
                if (TC_AppendString(szDomainAndUserName, MAX_PATH,
                  pszUserName)) {
                    pszUserName = szDomainAndUserName;
                }
            }
        }
    }

    TC_ZeroMemory(&nr, sizeof(nr));
    nr.dwType = RESOURCETYPE_DISK;
    nr.lpLocalName = pszLocalName;
    nr.lpRemoteName = pszRemoteName;

    dwErrorCode = WNetAddConnection2(&nr, pszPassword, pszUserName, 0);
    if (dwErrorCode != NO_ERROR) {
        TC_DbgPrintf(
          TEXT("WNetAddConnection2(%s, %s, %s) failed (ErrorCode=%d)\n"),
          pszLocalName, pszRemoteName, pszUserName, dwErrorCode);
    }

    return dwErrorCode;
}

DWORD WINAPI TCG_UnmapNetworkDrive(LPCTSTR pszLocalName)
{
    DWORD dwErrorCode;

    TC_Assert(pszLocalName);

    if (*pszLocalName == TEXT('\0')) return NO_ERROR;

    dwErrorCode = WNetCancelConnection2(pszLocalName, 0, TRUE);
    if (dwErrorCode != NO_ERROR) {
        TC_DbgPrintf(
          TEXT("WNetCancelConnection2(%s) failed (ErrorCode=%d)\n"),
          pszLocalName, dwErrorCode);
    }

    return dwErrorCode;
}

#endif
